/*******************************************************************************
* FILE NAME: user_routines.c <EDU VERSION>
*
* DESCRIPTION:
*  This file contains the default mappings of inputs  
*  (like switches, joysticks, and buttons) to outputs on the EDU RC.  
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
*******************************************************************************/

#include <stdio.h>
#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "serial_ports.h"
#include "eeprom.h"
#include "camera.h"
#include "camera_menu.h"
#include "tracking.h"
#include "tracking_menu.h"
#include "terminal.h"

void Camera_Turn(void);

#define BUTTON_REV_THRESH       100  // For the vex transmitter 5 & 6 buttons
#define BUTTON_FWD_THRESH       154

/*******************************************************************************
* FUNCTION NAME: Setup_Who_Controls_Pwms
* PURPOSE:       Each parameter specifies what processor will control the pwm.  
*                 
* CALLED FROM:   User_Initialization
*     Argument             Type    IO   Description
*     --------             ----    --   -----------
*     pwmSpec1              int     I   USER/MASTER (defined in ifi_aliases.h)
*     pwmSpec2              int     I   USER/MASTER
*     pwmSpec3              int     I   USER/MASTER
*     pwmSpec4              int     I   USER/MASTER
*     pwmSpec5              int     I   USER/MASTER
*     pwmSpec6              int     I   USER/MASTER
*     pwmSpec7              int     I   USER/MASTER
*     pwmSpec8              int     I   USER/MASTER
* RETURNS:       void
*******************************************************************************/
static void Setup_Who_Controls_Pwms(int pwmSpec1,int pwmSpec2,int pwmSpec3,int pwmSpec4,
                                    int pwmSpec5,int pwmSpec6,int pwmSpec7,int pwmSpec8)
{
  txdata.pwm_mask = 0xFF;         /* Default to master controlling all PWMs. */
  if (pwmSpec1 == USER)           /* If User controls PWM1 then clear bit0. */
    txdata.pwm_mask &= 0xFE;      /* same as txdata.pwm_mask = txdata.pwm_mask & 0xFE; */
  if (pwmSpec2 == USER)           /* If User controls PWM2 then clear bit1. */
    txdata.pwm_mask &= 0xFD;
  if (pwmSpec3 == USER)           /* If User controls PWM3 then clear bit2. */
    txdata.pwm_mask &= 0xFB;
  if (pwmSpec4 == USER)           /* If User controls PWM4 then clear bit3. */
    txdata.pwm_mask &= 0xF7;
  if (pwmSpec5 == USER)           /* If User controls PWM5 then clear bit4. */
    txdata.pwm_mask &= 0xEF;
  if (pwmSpec6 == USER)           /* If User controls PWM6 then clear bit5. */
    txdata.pwm_mask &= 0xDF;
  if (pwmSpec7 == USER)           /* If User controls PWM7 then clear bit6. */
    txdata.pwm_mask &= 0xBF;
  if (pwmSpec8 == USER)           /* If User controls PWM8 then clear bit7. */
    txdata.pwm_mask &= 0x7F;
}

/*******************************************************************************
* FUNCTION NAME: User_Initialization
* PURPOSE:       This routine is called first (and only once) in the Main function.  
*                You may modify and add to this function.
*                The primary purpose is to set up the DIGITAL IN/OUT - ANALOG IN
*                pins as analog inputs, digital inputs, and digital outputs.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Initialization (void)
{
/* FIRST: Set up the pins you want to use as analog INPUTs. */
//  IO1 = IO2 = INPUT;        /* Used for analog inputs. */
    /* 
     Note: IO1 = IO2 = IO3 = IO4 = INPUT; 
           is the same as the following:

           IO1 = INPUT;
           IO2 = INPUT;
           IO3 = INPUT;
           IO4 = INPUT;
    */

/* SECOND: Configure the number of analog channels. */
  Set_Number_of_Analog_Channels(NO_ANALOG);     /* See ifi_aliases.h */

/* THIRD: Set up any extra digital inputs. */
  /* The six INTERRUPTS are already digital inputs. */
  /* If you need more then set them up here. */
  /* IOxx = IOyy = INPUT; */
  IO1 = IO2 = INPUT;        /* Used for analog inputs. */
  IO6 = IO8 = IO10 = INPUT;      /* Used for limit switch inputs. */
  IO12 = IO14 = IO16 = INPUT;    /* Used for limit switch inputs. */
  IO3 = IO4 = INPUT;
  IO5 = IO7 = IO9 = INPUT;     /* For connecting to adjacent limit switches. */
  IO11 = IO13 = IO15 = INPUT;  /* For connecting to adjacent limit switches. */

/* FOURTH: Set up the pins you want to use as digital OUTPUTs. */
//  IO3 = IO4 = OUTPUT;
//  IO5 = IO7 = IO9 = OUTPUT;     /* For connecting to adjacent limit switches. */
//  IO11 = IO13 = IO15 = OUTPUT;  /* For connecting to adjacent limit switches. */

/* FIFTH: Initialize the values on the digital outputs. */
//  rc_dig_out03 = rc_dig_out04 = 0;
//  rc_dig_out05 = rc_dig_out07 = rc_dig_out09 = 0;
//  rc_dig_out11 = rc_dig_out13 = rc_dig_out15 = 0;

/* SIXTH: Set your initial PWM values.  Neutral is 127. */
//  pwm01 = 114; // 114 is centered
//  pwm02 = 70; // 70 is centered, 0 is looking down, 132 is looking up
  pwm01 = pwm02 = 127;
  pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;

/* SEVENTH: Choose which processor will control which PWM outputs. */
  Setup_Who_Controls_Pwms(MASTER,MASTER,MASTER,MASTER,MASTER,MASTER,MASTER,MASTER);

/* EIGHTH: Set your PWM output type.  Only applies if USER controls PWM 1, 2, 3, or 4. */
  /*   Choose from these parameters for PWM 1-4 respectively:                          */
  /*     IFI_PWM  - Standard IFI PWM output generated with Generate_Pwms(...)          */
  /*     USER_CCP - User can use PWM pin as digital I/O or CCP pin.                    */
  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

/* Add any other user initialization code here. */

  Init_Serial_Port_One();
  Init_Serial_Port_Two();

#ifdef TERMINAL_SERIAL_PORT_1    
  stdout_serial_port = SERIAL_PORT_ONE;
#endif

#ifdef TERMINAL_SERIAL_PORT_2    
  stdout_serial_port = SERIAL_PORT_TWO;
#endif

  Putdata(&txdata);             /* DO NOT CHANGE! */

  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */
}

/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Master_uP
* PURPOSE:       Executes every 17ms when it gets new data from the master 
*                microprocessor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Master_uP(void)
{
	static unsigned char count = 0;
	static unsigned char camera_menu_active = 0;
	static unsigned char tracking_menu_active = 0;
	unsigned char terminal_char;
	unsigned char returned_value;

	Getdata(&rxdata);

	// send diagnostic information to the terminal, but don't 
	// overwrite the camera or tracking menu if it's active
	if(camera_menu_active == 0 && tracking_menu_active == 0)
	{
		Tracking_Info_Terminal();
	}

	// This function is responsable for camera initialization 
	// and camera serial data interpretation. Once the camera
	// is initialized and starts sending tracking data, this 
	// function will continuously update the global T_Packet_Data 
	// structure with the received tracking information.
	Camera_Handler();

	// This function reads data placed in the T_Packet_Data
	// structure by the Camera_Handler() function and if new
	// tracking data is available, attempts to keep the center
	// of the tracked object in the center of the camera's
	// image using two servos that drive a pan/tilt platform.
	// If the camera doesn't have the object within it's field 
	// of view, this function will execute a search algorithm 
	// in an attempt to find the object.
	if(tracking_menu_active == 0)
	{
		Servo_Track();
	}

	// this logic guarantees that only one of the menus can be
	// active at any giiven time
	if(camera_menu_active == 1)
	{
		// This function manages the camera menu functionality,
		// which is used to enter camera initialization and
		// color tracking parameters.
		camera_menu_active = Camera_Menu();
	}
	else if(tracking_menu_active == 1)
	{
		// This function manages the tracking menu functionality,
		// which is used to enter parameters that describe how
		// the pan and tilt servos will behave while in searching
		// and tracking modes.
		tracking_menu_active = Tracking_Menu();
	}
	else
	{
		// has the user sent any data via the terminal?
		terminal_char = Read_Terminal_Serial_Port();
		// check to see if any "hotkeys" have been pressed
		if(terminal_char == CM_SETUP_KEY)
		{
			camera_menu_active = 1;
		}
		else if(terminal_char == TM_SETUP_KEY)
		{
			tracking_menu_active = 1;
		}
	}

	// This funtion is used by the functions Camera_Menu() and
	// Tracking_Menu() to manage the writing of initialization
	// parameters to your robot controller's non-volatile
	// Electrically Erasable Programmable Read-Only Memory
	// (EEPROM)
	EEPROM_Write_Handler();

//    Default_Routine();
//	if (PWM_in5 < BUTTON_REV_THRESH)
//  	{
//		Camera_Turn();
//  	}

	Camera_Turn();  // Keep the robot facing the camera

	Putdata(&txdata);
}


/*******************************************************************************
* FUNCTION NAME: Camera_Turn
* PURPOSE:       Turn the robot to face the target
*                Assume the servo powered gimbal is much faster than moving the whole robot
*******************************************************************************/
#define CENTER   127   // The servo setting when the robot is pointed at the target
#define KP     12/10   // Proportional gain
#define KI     5/100    // Integral gain (if a fraction then take care with MAXERROR)
#define DEADBAND 2
#define NEUTRAL  127
#define TARGET_CONFIDENCE_THRESHOLD 150
#define MAXERROR 300  // Maximum error we will allow to accumulate (MAXERROR < SPEED_LIMIT/KI)
#define SPEED_LIMIT 60

void Camera_Turn(void)
{
  static int cumerror=0;
  signed int pan_error;
  signed int p, i;
  signed int output;


  if (T_Packet_Data.confidence > TARGET_CONFIDENCE_THRESHOLD)  // First can we see the target?
  {
    pan_error = CENTER - (signed int)PAN_SERVO;  // How far off the target are we?
    if((pan_error <= DEADBAND) && (pan_error >= -DEADBAND) )
    { // We're there
	  pwm07 = NEUTRAL;
	  pwm08 = NEUTRAL;
      cumerror = 0;  // Can zero this to stop over-correcting
//      printf("We're done\r");
    }
    else    // Keep turning towards the light
    { 

      // Apply power Proportional to how far off we are
      p =  pan_error * KP;

      // Integral - Correct if we've been off by a little bit or if it's taking us too long
      i = cumerror * KI;
      cumerror += pan_error;
      if (cumerror > MAXERROR) cumerror = MAXERROR;
	  if (cumerror < -MAXERROR) cumerror = -MAXERROR;

      output = p + i;  // Calculate for motor forward
	  // Limit the motor output so it doesn't whip around too fast & isn't > char

      if (output > SPEED_LIMIT) output = SPEED_LIMIT;
      if (output < -SPEED_LIMIT) output = -SPEED_LIMIT;
      pwm07 = NEUTRAL - (unsigned char) output;
      pwm08 = NEUTRAL - (unsigned char) output; // The motors on one side are reversed
    }
//    printf("pan_error = %d, cum_pan_err, %d, PAN_SERVO = %d, pwm07=%d, pwm08=%d, output=%d\r",
//			(signed int)pan_error,cumerror,PAN_SERVO,pwm07,pwm08,output);
  }
  else  // Haven't found the target
  { // Wait for the camera to acquire the target (could spin the robot 360)
	pwm07 = NEUTRAL;
	pwm08 = NEUTRAL;	
//    printf("Haven't seen the target yet\r");
  }
}


/*******************************************************************************
* FUNCTION NAME: Default_Routine
* PURPOSE:       Performs the default mappings of inputs to outputs for  
*                CONFIGURATIONS A, B, and C (B = drive mode 12 - settings on radio)
* CALLED FROM:   this file, Process_Data_From_Master_uP routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Default_Routine(void)
{

//*** 1-joystick driving ********
  pwm07 = PWM_in2;
  pwm08 = 255 - PWM_in3;

} /* END Default_Routine(); */
